<!DOCTYPE HTML>
<%@ taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@ page import="org.osgi.framework.BundleContext" %>
<%@ page import="org.osgi.framework.ServiceReference" %>
<%@ page import="de.virtimo.bpc.api.service.CoreBundleService" %>
<%@ page import="de.virtimo.bpc.api.exception.*" %>
<%@ page import="de.virtimo.bpc.api.SystemException" %>
<%@ page import="java.util.Map" %>
<%@ page import="java.util.HashMap" %>
<%@ page import="java.util.logging.Logger" %>
<%@ page import="java.util.logging.Level" %>
<%
    boolean bpcShowError = false;
    boolean bpcBackendProblem = false;
    boolean userFlowProviderError = false;
    try {
        BundleContext ctx = null;
        ServiceReference ref = null;
        try {
            ctx = (BundleContext) getServletContext().getAttribute("osgi-bundlecontext");
            ref = ctx.getServiceReference(CoreBundleService.class.getName());
            CoreBundleService coreBundleService = (CoreBundleService) ctx.getService(ref);

            // redirect the user to the login page, but only when:
            // - keycloak/oidc is the used identity provider
            // - and it is not a "login done callback" from the identity provider
            // - and there is no valid user session
            if (coreBundleService.isUserFlowIdentityProviderInUse() && !coreBundleService.isUserFlowIdentityProviderCallback(request)) {
                if (!coreBundleService.hasValidUserSession(request)) {
                    String redirectUrl = coreBundleService.getUserFlowIdentityProviderLoginRedirectUrl(request);
                    out.write("<html>");
                    out.write("<head>");
                    out.write("<meta charset=\"UTF-8\">");
                    out.write("<title>Login Page Redirection</title>");
                    out.write("<script type=\"text/javascript\" src=\"resources/index-scripts/redirect.js\"></script>");
                    out.write("</head>");
                    out.write("<body data-redirect-url=\"" + redirectUrl + "\">");
                    out.write("If you are not redirected automatically, follow this <a href=\"" + redirectUrl + "\">link to login</a>.");
                    out.write("</body>");
                    out.write("</html>");
                    return;
                }
            }

            // get the theme config
            Map<String, Object> themeConfig = coreBundleService.getThemeConfig(request);
            pageContext.setAttribute("themeConfig", themeConfig);
        } finally {
            if (ctx != null && ref != null) {
                ctx.ungetService(ref);
            }
        }
    } catch (UserFlowIdentityProviderResponseException ex) {
        bpcShowError = true;
        userFlowProviderError = true;

        Map<String, String> userFlowProviderErrorData = new HashMap<>();
        userFlowProviderErrorData.put("error", ex.getError());
        userFlowProviderErrorData.put("errorDescription", ex.getErrorDescription());
        pageContext.setAttribute("userFlowProviderErrorData", userFlowProviderErrorData);
    } catch (Exception ex) {
        Logger.getLogger("index.jsp").log(Level.SEVERE, "index.jsp exception", ex);
        bpcShowError = true;
        bpcBackendProblem = true;
    }
    // values of java variables must be stored as a page attribute, otherwise the JSP-tags are not able to access the value!
    pageContext.setAttribute("bpcShowError", bpcShowError);
    pageContext.setAttribute("bpcBackendProblem", bpcBackendProblem);
    pageContext.setAttribute("userFlowProviderError", userFlowProviderError);
%>
<html>
<head>
    <%-- Use the config to define title, if no definition is found, use the standard html  --%>
    <c:choose>
        <c:when test="${not empty themeConfig['appTitle']}">
            <title><c:out value="${themeConfig['appTitle']}"/></title>
        </c:when>
        <c:otherwise>
            <title>Virtimo Business Process Center</title>
        </c:otherwise>
    </c:choose>
    <!-- Robots-Meta-Tag forbidding crawler to index this site or follow links on it-->
    <meta name="robots" content="noindex,nofollow" />
    <meta name="googlebot" content="noindex,nofollow" />
    <!-- html5 -->
    <meta charset="utf-8"/>
    <!-- html4 -->
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
    <!-- ie edge compatibility mode / use latest renderer -->
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no">
    <%-- Use the config to define description, if no definition is found, use the standard  --%>
    <c:choose>
        <c:when test="${not empty themeConfig['appMetaDescription']}">
            <meta name="description" content="<c:out value="${themeConfig['appMetaDescription']}"/>">
        </c:when>
        <c:otherwise>
            <meta name="description" content="Digitale Transformation - Das Virtimo Business Process Center als durchgängige Plattform für webbasierte, energiewirtschaftliche Geschäftsanwendungen und Portallösungen.">
        </c:otherwise>
    </c:choose>
    <%-- Use the config to define favicon, if no definition is found, use the standard  --%>
    <c:choose>
        <c:when test="${ not empty themeConfig['appFavIcon'] }">
            <link rel="shortcut icon" type="image/x-icon" href="<c:out value="${themeConfig['appFavIcon']}?"/>">
        </c:when>
        <c:otherwise>
            <link rel="icon" href="resources/images/RZ_Virtimo_Labels_RGB_16x16px_BPC.png">
        </c:otherwise>
    </c:choose>



    <style>
        body {
            background-size: cover !important;
            background-repeat: no-repeat !important;
            transition: background-image 3s ease-in-out;
            margin: 0;
            color: rgba(0,0,0,.87);
        }

        body, html {
            height: 100%;
        }

        #loader-wrapper {
            position: fixed;
            top: 0;
            left: 0;
            width: 100%;
            height: 100%;
            z-index: 1000;
        }

        #loadText {
            text-align: center;
            width: 1000px;
            left: 0;
            right: 0;
            margin: auto;
            position: absolute;
            top: 65%;
            color: #677488;
            background: linear-gradient(to right, transparent, rgba(220, 218, 210, 0.8), transparent);
            border-radius: 15px;
        }

        #loader-icon {
            display: flex;
            position: relative;
            top: 50%;
            margin-top: -110px;
        }

        #loader-icon img {
            margin: auto;
            width: 300px;
        }

    <%--Start Styling for Error Page--%>
        body.desaturate:before {
            content: '';
            position: absolute;
            top: 0;
            right: 0;
            bottom: 0;
            left: 0;
            background: inherit;
            /*grayscale for background image*/
            -webkit-filter: grayscale(100%);
            filter: grayscale(100%);
            z-index:-1;
        }

    .page-header{
        height: 48px;
        width: 100%;
        background: rgba(220, 217, 210, 0.8);
        display: flex;
        z-index: 1;
        font-family: 'Barlow', sans-serif;
    }
    .page-header .company-logo{
        background-image: url('resources/images/logo.png');
        background-repeat: no-repeat;
        background-position: center;
        background-size: contain;
        height: 100%;
        width: 190px;
        margin-left: 16px;
    }
    .page-header .page-title{
        width: 100%;
        height: 100%;
        font-size: 14px;
        display: flex;
        align-items: center;
        justify-content: center;
    }
    .page-content{
        position: absolute;
        top:0;
        left:0;
        bottom: 0;
        right: 0;
        display: flex;
        align-items: center;
        justify-content: center;
    }

    .page-content-inner-container{
        display: flex;
        flex-direction: column;
        font-family: 'Barlow', sans-serif;
        max-width: 840px;
        width: 60%;
        background: rgba(255, 255, 255, 0.7);
        box-shadow: rgba(0, 0, 0, 0.1) 2px 2px 8px;
        backdrop-filter: blur(5px);
    }

    .page-content-inner-container .message-header{
        height: 48px;
        width: 100%;
        font-size: 14px;
        padding: 16px 32px;
        box-sizing: border-box;
        box-sizing: border-box;
        background: #fff;
        border-bottom: 1px solid rgba(0,0,0,.1);
    }
    .page-content-inner-container .message-body{
        padding: 32px;
        width: 100%;
        height: 100%;
        box-sizing: border-box;
    }

    .page-content-inner-container h1{
        font-size: 36px;
        text-align: center;
        font-weight: normal;
        color: #EE5252;
    }
    <%--End Styling for Error Page--%>



    </style>
</head>
<c:choose>
    <c:when test="${bpcShowError == true}">
        <body class="desaturate">
            <div class="page-header">
            <%--<div class="company-logo"></div>--%>
                <div class="page-title">Error Page</div>

            </div>
            <div class="page-content">
                 <div class= "page-content-inner-container">
                    <div class="message-header">Message: </div>

                    <div class="message-body">
                        <c:choose>
                            <c:when test="${bpcBackendProblem == true}">
                                <h1>Backend not reachable. Please try again later. Thanks!</h1>
                            </c:when>
                            <c:when test="${userFlowProviderError == true}">
                                <h1>User flow provider error. Please check your IdP configuration. Thanks!</h1>
                                <c:choose>
                                    <c:when test="${not empty userFlowProviderErrorData['errorDescription']}">
                                        <h2><c:out value="${userFlowProviderErrorData['errorDescription']}"/></h2>
                                    </c:when>
                                    <c:otherwise>
                                        <h2><c:out value="${userFlowProviderErrorData['error']}"/></h2>

                                        //window.history.replaceState(null,null,window.location.protocol + '//' + window.location.host + window.location.pathname)

                                    </c:otherwise>
                                </c:choose>
                            </c:when>
                            <c:otherwise>
                                <h1>Unknown error state!</h1>
                            </c:otherwise>
                        </c:choose>
                    </div>
                 </div>
            </div>
            <link href="https://fonts.googleapis.com/css2?family=Lato&display=swap" rel="stylesheet">
        </body>
    </c:when>
    <c:otherwise>
        <body>
            <div id="loader-wrapper">
        <%-- Use the config to define loader icon, if no definition is found, use the standard  --%>
            <c:choose>
                <c:when test="${not empty themeConfig['appLoaderIcon']}">
                    <div id="loader-icon" style="display: none;">
                    <c:choose>
                        <c:when test="${not empty themeConfig['appLoaderIconWidth']}">
                            <img style="width:<c:out value="${themeConfig['appLoaderIconWidth']}"/>;" src="<c:out value="${themeConfig['appLoaderIcon']}"/>">
                        </c:when>
                        <c:otherwise>
                            <img alt="loader icon" src="<c:out value="${themeConfig['appLoaderIcon']}"/>">
                        </c:otherwise>
                    </c:choose>
                    </div>
                </c:when>
                <c:otherwise>
                    <div id="loader-icon" style="display: none;">
                        <img src="resources/images/load_animation.svg">
                    </div>
                </c:otherwise>
            </c:choose>
                <h3 id="loadText">Initialisiere BPC Framework</h3>
            </div>

            <!-- The line below must be kept intact for Sencha Cmd to build your application -->
            <script  id="microloader" data-app="92fe03d8-6f49-4c0e-b989-73c408e18a1b" src="microloader.js"></script>
        </body>

        <script type="text/javascript" src="resources/index.js"></script>
    </c:otherwise>
</c:choose>
<c:choose>
    <c:when test="${bpcShowError == true && userFlowProviderError == true}">
        <script type="text/javascript" src="resources/index-scripts/userFlowProviderError.js"></script>
    </c:when>
</c:choose>
</html>
